/// <summary>
/// Access (DRM) Policy is at the root of controlling end users' access to protected content. 
/// <b>IMPORTANT NOTE</b>: Policies can exist at multiple levels (global, content). When multiple policies apply, Vitrium uses 
/// a most lenient combination of applicable Access Policy settings in an attempt to grant access to protected content. When a particular field validation
/// fails for one policy, it may pass for another and grant access to content. It is important to be aware of this concept when combining Access Policies
/// at different levels as it may lead to authorization results which are difficult to trace. It is advisable to only use AccessPolicies at a single chosen
/// level and set irrelevant fields to Unlimited (generally null but documented for each setting).
/// </summary>
class AccessPolicy {
    Name; // string
    ComputersMax; // int
    PdfLimit; // int
    BrowserLimit; // int
    IpAddressesMax; // int
    IgnoredIpAddresses;
    Expiry;  // date
    RelativeExpiryInDay; // int
    OfflineDurationinDays; // int
    StartDate; // date
    DocumentLimit; // int
    OpenLimit; // int
    PdfLimit; // int
    WebPrintLimit; // int
    AnnotationsAllowed; // bool
    SetAsGlobal; // bool
    LocationRestrictions; // string
    LocationPermits; // string
    AllowDownloadSourceFile; // bool
    CustomField; /// Optional string which can store any value for your internal use.

    constructor(name) // constructor can be taken out
    {  
        this.name = name; /// Unique Access Policy name. Required in all API calls except UniqueDoc's AccessPolicyOverride.
    }
}

class WebViewerDocPolicy {
    AllowAnnotations = false; // bool
    AllowCopy = false; // bool
    AllowPrint = false; // bool
    AllowWebPrint = false; // bool

    constructor(AllowAnnotations, AllowCopy, AllowPrint, AllowWebPrint)
    {  
        this.AllowAnnotations = AllowAnnotations;
        this.AllowCopy = AllowCopy;
        this.AllowPrint = AllowPrint;
        this.AllowWebPrint = AllowWebPrint;
    }
}

module.exports.AccessPolicy = AccessPolicy;
module.exports.WebViewerDocPolicy = WebViewerDocPolicy;