var express = require('express');
const { Helpers } = require('../Helpers');
const { DataAccess } = require('../DataAccess');

var router = express.Router();

router.get('/', (req, res) => {
   console.log('GET authenticate endpoint called');
   res.send("~This is an example of JSON API v3 it allows you to get a direct authentication and authorization within one request.");
});


function handleAuthentication(req) {
   var results;

   console.log("handleAuthentication", req);

   console.log('req type is: ' + req.Type);

   switch (req.Type) {
      case "UserCredentials": // username, password, and document metadata are passed in
      case "UniqueDocCopyIdToken": // you need to create a document with VersionUnique, download to print uses this as well
      case "PrintMeteringUsernameToken": // the webviewer print
      case "WebViewerSessionTokenVerification": // when you leave your webviewer on for 4 or 5 minutes, this one must be implemented
      case "PhoneUnlockToken": // offline unlock, in the contents tab, has an offline unlock button
         console.log("handleAuthentication 1");
         results = processRequest(req);
         console.log('results were ' , results);
         break;
      case "HashedUserCredentials": //Username, Password, HashingKey, HashingVersion, CaseSensitivePassword // This is an endpoint for an old unused scenario.
         console.log("handleAuthentication 2");
         results = Helpers.generateUnsuccessfulAuthenticationResult("HashedUserCredentials request is not supported by this implementation of External Service V3");
         break;
      case "SsoLiteToken": //Token is User Id, Document, if you open up a PDF in Adobe, unlock it without any offline days, and then open it again
         console.log("handleAuthentication 3");
         results = processRequestTokenisUserId(req);
         break;
      case "WebViewerSso": //Token is token, Document, if you open up a PDF in the WebViewer, unlock it without any offline days, and then open it again
         console.log("handleAuthentication 4");
         results = processRequestTokenisToken(req);
         break;
      default:
         console.log("handleAuthentication 5 default option in handleAuthentication");
         results = Helpers.generateUnsuccessfulAuthenticationResult(req.Type + " request is not supported");
         break;
   }

   return results;
}


function processRequest(request) {
   // We search from most specific to least specific
   if (request.Username !== null && request.Password !== null && request.Document !== null) //Case 1: All 3 fields have value
   {
      console.log('processRequest 1 ', request.CaseSensitivePassword);
      
      return DataAccess.getAuthenticationUDCD(request.Username, request.Password, request.CaseSensitivePassword, request.Document);
   }

   if (request.Username !== null && request.Password !== null && request.Document === null) //Case 2: document is null
   {
      console.log('processRequest 2');
      var reader = DataAccess.getReaderByUsernameAndPassword(request.Username, request.Password, request.CaseSensitivePassword);
      if (reader === null) {
         return Helpers.generateUnsuccessfulAuthenticationResult(null);
      }
      return Helpers.generateSuccessfulAuthenticationResult(reader);
   }
 
   if (request.Username !== null && request.Password === null && request.Document !== null) //Case 3: password is null
   {
      console.log('processRequest 3');
      return DataAccess.getAuthenticationUDCD(request.Username, null, null, request.Document);
   }

   if (request.Username !== null) {
      console.log('processRequest 4');
      var reader = DataAccess.getReaderByUsername(request.Username);
      if (reader === null) {
         return Helpers.generateUnsuccessfulAuthenticationResult("Your account could not be found in the system. Make sure you are entering the correct details (3yq)");
      }
      return Helpers.generateSuccessfulAuthenticationResult(reader);
   }
}

function processRequestTokenisUserId(req) {
   if (req.Token !== null && req.Document !== null) {
      return DataAccess.getAuthenticationById(req.Token, req.Document);
   }
   else if (request.Token !== null && request.Document === null) {
      var reader = DataAccess.getReaderById(request.Token);
      if (reader != null) {
         return Helpers.generateSuccessfulAuthenticationResult(reader);
      }
   }

   return Helpers.generateUnsuccessfulAuthenticationResult(null);
}

function processRequestTokenisToken(req) {
   if (req.Token !== null && req.Document !== null) {
      return DataAccess.getAuthenticationByToken(req.Token, req.Document);
   }
   else if (req.Token !== null && req.Document === null) {
      var reader = DataAccess.getReaderByToken(req.Token);
      if (reader !== null) {
         return Helpers.generateSuccessfulAuthenticationResult(reader);
      }
   }

   return Helpers.generateUnsuccessfulAuthenticationResult(null);
}


router.post('/', (req, res) => {

   console.log('POST authenticate endpoint called', JSON.stringify(req.body));
   var response = handleAuthentication(req.body);
   res.setHeader('Content-Type', 'application/json');

   console.log('sending response', response);
   res.send(JSON.stringify(response));
});

module.exports = { router, handleAuthentication };